#include <Box.h>
#include <Directory.h>
#include <Entry.h>
#include <Rect.h>
#include <MenuItem.h>
#include <Path.h>

#include <string.h>
#include <stdlib.h>

#include "PrefMessages.h"
#include "ModuleGui.h"

ModuleGui::ModuleGui(BView *pc_ParentView, uint32 i_Top)
{
   int i;
   BBox *pc_ModuleBox;
   BRect c_ViewRect;
   TS_Driver   *ps_Driver;

   pas_Modules = (TS_Module*) malloc(sizeof(TS_Module)*MODULE_MAX_MODULE);
   InitModule();
   
   c_ViewRect = pc_ParentView->Bounds();
   c_ViewRect.left += 5;
   c_ViewRect.right -= 5;
   c_ViewRect.top = i_Top + 5;
   c_ViewRect.bottom = i_Top + 80;
   
   pc_ModuleBox = new BBox(c_ViewRect);
   pc_ModuleBox->SetLabel("Module/Driver");
   pc_ParentView->AddChild(pc_ModuleBox);

   /* module MenuField */
   pc_ModuleMenu = new BPopUpMenu("ModuleMenu");
    
   BMenuItem *item;
   for (i = 0;i < MODULE_MAX_MODULE;i++)
   {
      if(pas_Modules[i].name[0] != '\0')
      {
         pc_ModuleMenu->AddItem(item = new BMenuItem(pas_Modules[i].name,
                                new BMessage(PREF_MSG_MODULE_CHANGED)));
         if (i == 0)
            item->SetMarked(true);
      }
   }
   c_ViewRect = pc_ModuleBox->Bounds();
   c_ViewRect.InsetBy(5,15);
   pc_ModuleMenuField= new BMenuField(c_ViewRect ,"Module", B_EMPTY_STRING, pc_ModuleMenu);
   pc_ModuleMenuField->SetDivider(0.0);
   pc_ModuleBox->AddChild(pc_ModuleMenuField);

   /* driver MenuField */
   pc_DriverMenu = new BPopUpMenu("DriverMenu");
   ps_CurModule = &(pas_Modules[0]);
    
   for (i = 0;i < MODULE_MAX_DRIVER;i++)
   {
      ps_Driver = &(pas_Modules[0].driver[i]);
      if(ps_Driver->name[0] != '\0')
      {
         pc_DriverMenu->AddItem(ps_Driver->menu = new BMenuItem(pas_Modules[0].driver[i].name,
                              new BMessage(PREF_MSG_DRIVER_CHANGED)));
      }
      if (i == 0)
         ps_Driver->menu->SetMarked(true);
   }
   c_ViewRect = pc_ModuleBox->Bounds();
   c_ViewRect.InsetBy(5,45);
   pc_DriverMenuField= new BMenuField(c_ViewRect ,"Driver", B_EMPTY_STRING, pc_DriverMenu);
   pc_DriverMenuField->SetDivider(0.0);
   pc_ModuleBox->AddChild(pc_DriverMenuField);

   return;
}

ModuleGui::~ModuleGui(void)
{
   if(pas_Modules != NULL)
      free(pas_Modules);
   return;
}

status_t ModuleGui::InitModule(void)
{
   uint8       i_CurModule, i_CurDriver;
   status_t    i_ret;
   BDirectory  *pc_Directory;
   BPath       c_Path;
   BEntry      c_Entry;

   /* empty all entries */
   for(i_CurModule = 0; i_CurModule < MODULE_MAX_MODULE; i_CurModule++)
   {
      pas_Modules[i_CurModule].name[0] = '\0';
      for(i_CurDriver = 0; i_CurDriver < MODULE_MAX_DRIVER; i_CurDriver++)
      {
         pas_Modules[i_CurModule].driver[i_CurDriver].name[0] = '\0';
         pas_Modules[i_CurModule].driver[i_CurDriver].menu = NULL;
      }
   }
      
   /* initialize modules names */
   strcpy(pas_Modules[0].name, "default");
   strcpy(pas_Modules[1].name, "irman");

   /* initialize default drivers */
   pc_Directory = new BDirectory("/dev/misc/berc");
   i_CurDriver = 0;
   if(pc_Directory != NULL)
   {
      i_ret = pc_Directory->GetEntry(&c_Entry);
      while(i_ret == B_OK)
      {
         c_Entry.GetPath(&c_Path) ;
         if(strstr(c_Path.Path(), "serial") != NULL)
         {
            strcpy(pas_Modules[0].driver[i_CurDriver].name, c_Path.Path());
            i_CurDriver++;
         }
         i_ret = pc_Directory->GetNextEntry(&c_Entry);
      }
      delete pc_Directory;
   }
   
   /* initialize irman drivers */
   pc_Directory = new BDirectory("/dev/ports");
   i_CurDriver = 0;
   if(pc_Directory != NULL)
   {
      i_ret = pc_Directory->GetEntry(&c_Entry);
      while(i_ret == B_OK)
      {
         c_Entry.GetPath(&c_Path) ;
         if(strstr(c_Path.Path(), "serial") != NULL)
         {
            strcpy(pas_Modules[1].driver[i_CurDriver].name, c_Path.Path());
            i_CurDriver++;
         }
         i_ret = pc_Directory->GetNextEntry(&c_Entry);
      }
      delete pc_Directory;
   }

   return(B_OK);
}
   
status_t ModuleGui::UpdateDriverList(void)
{
   uint8 i_CurIndex, i_ModuleIndex;
   int32 i_PrevLastIndex;
   TS_Driver   *ps_Driver;
 
   i_CurIndex = 0;
   i_ModuleIndex = pc_ModuleMenuField->Menu()->IndexOf(pc_ModuleMenuField->Menu()->FindMarked());
   if(ps_CurModule == &pas_Modules[i_ModuleIndex])
      return(B_OK);  /* no change */
   
   /* Add the new menu items */
   for (i_CurIndex = 0;i_CurIndex < MODULE_MAX_DRIVER; i_CurIndex++)
   {
      ps_Driver = &(pas_Modules[i_ModuleIndex].driver[i_CurIndex]);
      if(ps_Driver->name[0] != '\0')
      {
         pc_DriverMenu->AddItem(ps_Driver->menu = new BMenuItem(pas_Modules[i_ModuleIndex].driver[i_CurIndex].name,
                                 new BMessage(PREF_MSG_DRIVER_CHANGED)));
      }
      if (i_CurIndex == 0)
         ps_Driver->menu->SetMarked(true);
   }

   /* delete the old ones */
   for (i_CurIndex = 0;i_CurIndex < MODULE_MAX_DRIVER; i_CurIndex++)
   {
      if(ps_CurModule->driver[i_CurIndex].menu != NULL)
      {
         pc_DriverMenu->RemoveItem(ps_CurModule->driver[i_CurIndex].menu);
         delete ps_CurModule->driver[i_CurIndex].menu;
         ps_CurModule->driver[i_CurIndex].menu = NULL;
      }
   }

   ps_CurModule = &pas_Modules[i_ModuleIndex];
   return(B_OK);
}

status_t ModuleGui::SetModule(char *psz_Module, char *psz_Driver)
{
   uint8       i_CurModule, i_CurDriver;
   status_t    i_ret;
   TS_Module   *ps_Module;
   TS_Driver   *ps_Driver;

   /* find and change module entry */
   for(i_CurModule = 0; i_CurModule < MODULE_MAX_MODULE; i_CurModule++)
   {
      ps_Module = &pas_Modules[i_CurModule];
   
      if(strcmp(ps_Module->name, psz_Module) == 0)
      {
         BMenuItem *pc_ModuleItem = pc_ModuleMenuField->Menu()->ItemAt(i_CurModule);
         if(pc_ModuleItem != NULL)
         {
            pc_ModuleItem->SetMarked(true);
            UpdateDriverList();

            /* search driver entry */
            for(i_CurDriver = 0; i_CurDriver < MODULE_MAX_DRIVER; i_CurDriver++)
            {
               ps_Driver = &(pas_Modules[i_CurModule].driver[i_CurDriver]);
               if(strcmp(ps_Driver->name, psz_Driver) == 0)
               {
                  BMenuItem *pc_DriverItem = pc_DriverMenuField->Menu()->ItemAt(i_CurDriver);
                  if(pc_DriverItem != NULL)  /* update the menus */
                  {
                     pc_DriverItem->SetMarked(true);
                     return(B_OK);
                  }
               }
            } /* for driver */
         } 
      }
   } /* for module */
   return(B_ERROR);
}

status_t ModuleGui::GetModule(char *psz_Module, char *psz_Driver)
{
   uint8 i_ModuleIndex, i_DriverIndex;
   
   i_ModuleIndex = pc_ModuleMenuField->Menu()->IndexOf(pc_ModuleMenuField->Menu()->FindMarked());
   i_DriverIndex = pc_DriverMenuField->Menu()->IndexOf(pc_DriverMenuField->Menu()->FindMarked());

   strcpy(psz_Module, pas_Modules[i_ModuleIndex].name);
   strcpy(psz_Driver, pas_Modules[i_ModuleIndex].driver[i_DriverIndex].name);

   return(B_OK);
}

