#include <ParameterWeb.h>
#include <TimeSource.h>

#include "AudioVolume.h"

/* Most of this code is stollen from stampTV */

AudioVolume::AudioVolume()
{
   status_t status = B_OK;

   /* find the media roster */
   pc_MediaRoster = BMediaRoster::Roster(&status);
   pc_VolumeBar = new VolumeBar();
}

AudioVolume::~AudioVolume()
{
   delete pc_VolumeBar;
}

status_t AudioVolume::SetAudioVolume(te_VolumeCmd e_Cmd, int32 i_Value)
{
   uint8 i_BarLevel;
   static uint8 i_LastBarLevel = 0;
   media_node  c_OutNode;
   status_t status = B_OK;
  
   /* check it again because when the filter is started from the boot, the
      media roster is not valid yet. Thus we initialize it here            */
   if(pc_MediaRoster == NULL)
      pc_MediaRoster = BMediaRoster::Roster(&status);

   i_BarLevel = 0;
   if(pc_MediaRoster && pc_MediaRoster->GetAudioOutput(&c_OutNode) == B_OK) 
   {
      BParameterWeb * web;
      
      if(pc_MediaRoster->GetParameterWebFor(c_OutNode, &web) == B_OK && web) 
      {
         BContinuousParameter *gain = NULL;

         for(int32 i = 0; i < web->CountParameters(); i++) 
         {
            BParameter *parameter = web->ParameterAt(i);
            
            if(parameter) 
            {
               BParameterGroup *group = parameter->Group();
               
               if( (e_Cmd == e_VolumeCmdLevel) &&
                   (strcmp(parameter->Kind(), B_GAIN) == 0) )
               {
                  gain = dynamic_cast<BContinuousParameter *> (parameter);
                  if (gain) 
                  {
                     float min = gain->MinValue();
                     float max = gain->MaxValue();
                     /* float step = gain->ValueStep() * float(i_Value); */
                     float step = float(i_Value) * ( (max-min) / NB_VOLUME_LEVEL);
                     int   channelsCount = gain->CountChannels();

                     if (channelsCount > 0) 
                     {
                        float       *channels = new float[channelsCount];
                        size_t      size = sizeof(float) * channelsCount;
                        bigtime_t   when;
                        
                        if(gain->GetValue(channels, &size, &when) == B_OK && (size > 0) )
                        {
                           int   count = size / sizeof(float);
                           for (int k = 0; k < count; k++) 
                           {
                              float v = channels[k];
                              v += step;
                              if (v > max)
                                 v = max;
                              else if (v < min)
                                 v = min;
                              channels[k] = v;
                           }
                           gain->SetValue(channels, size, BTimeSource::RealTime());
                           
                           if(i_BarLevel == 0)
                           {
                              i_BarLevel = (uint8)((channels[0]-min) * NB_VOLUME_LEVEL / (max-min));
                              i_LastBarLevel = i_BarLevel;
                           }
                        }
                        delete [] channels;
                     }
                  } /* if gain */
               }
               else if(strcmp(parameter->Kind(), B_MUTE) == 0) 
               {
                  uint32 i_MuteValue;
                  
                  if(e_Cmd != e_VolumeCmdMute)
                  {
                     i_MuteValue = 0;
                  }
                  else
                  {
                     i_MuteValue = i_Value;
                     if(i_Value == 0)
                        i_BarLevel = i_LastBarLevel;
                  }
                  
                  // mute selector: read its state
                  BDiscreteParameter * dp = dynamic_cast<BDiscreteParameter *> (parameter);
                  if (dp) 
                  {
                     int32 value;
                     bigtime_t time;
                     size_t size = sizeof(int32);
                     
                     if ((parameter->GetValue(reinterpret_cast<void *>(&value), &size, &time) == B_OK)
                           && (size <= sizeof(int32))) 
                     {
                        dp->SetValue(reinterpret_cast<void *>(&i_MuteValue), 
                                    sizeof(int32), BTimeSource::RealTime());
                     }
                  }  /* if dp */
               }  /* if MUTE parameter */
            }
         }  /* for each parameter */

         delete web;
      }
      pc_MediaRoster->ReleaseNode(c_OutNode);
   }

   pc_VolumeBar->SetVolumeLevel(i_BarLevel);

   return(B_OK);
}
