#include <OS.h>

#include <stdio.h>

#include "serial.h"

status_t SERIAL_InitDeviceArray(TS_SERIAL_device *ps_DeviceArray, 
                              TS_SERIAL_device *ps_DefaultValue)
{
   uint8 i;

   for(i=0; i<SERIAL_DEVICE_MAX; i++)
   {
      ps_DeviceArray[i] = *ps_DefaultValue;
      ps_DeviceArray[i].is_free = true;
      ps_DeviceArray[i].sense = -1;
      ps_DeviceArray[i].read_sem = create_sem(0, "Berc dev_sem");
      ps_DeviceArray[i].is_reading = false;
      if(ps_DeviceArray[i].read_sem < B_OK)         
         return(ps_DeviceArray[i].read_sem);
      
      ps_DeviceArray[i].wait_data_sem = create_sem(0, "Berc dev_sem");
      ps_DeviceArray[i].is_waiting_data = false;
      if(ps_DeviceArray[i].wait_data_sem < B_OK)         
      {
         delete_sem(ps_DeviceArray[i].read_sem);
         return(ps_DeviceArray[i].wait_data_sem);
      }
   }
   return(B_OK);
}

status_t SERIAL_FiniDeviceArray(TS_SERIAL_device *ps_DeviceArray)
{
   uint8 i;

   for(i=0; i<SERIAL_DEVICE_MAX; i++)
   {
      delete_sem(ps_DeviceArray[i].read_sem);
      delete_sem(ps_DeviceArray[i].wait_data_sem);
   }
   return(B_OK);
}

TS_SERIAL_device *SERIAL_NewDevice(TS_SERIAL_device *ps_DeviceArray)
{
   uint8 i;

   for(i=0; i<SERIAL_DEVICE_MAX; i++)
   {
      if(ps_DeviceArray[i].is_free == true)
      {
         ps_DeviceArray[i].is_free = false;
         if(ps_DeviceArray[i].name[0] == '\0')
            sprintf(ps_DeviceArray[i].name, "misc/berc/berc_serial%d", i);
         return(&(ps_DeviceArray[i]));
      }
   }
   return(NULL);
}

void SERIAL_FreeDevice(TS_SERIAL_device *ps_Device)
{
   ps_Device->is_free = true;
}
