#ifndef _SERIAL_H
#define _SERIAL_H

#include <drivers/KernelExport.h>
#include "serial_reg.h"

/************************
 *       Defines        *
 ************************/
#define LIRC_HOMEBREW        0
#define LIRC_IRDEO           1
#define LIRC_IRDEO_REMOTE    2
#define LIRC_ANIMAX          3

#define SERIAL_DEVICE_MAX  4
#define SERIAL_NAME_SIZE    32

#define DBG_PRINT(s) dprintf("BERC - SERIAL : %s\n", s)

/************************
 *       Structures     *
 ************************/
typedef struct 
{
   uint8    is_free;
   uint8    nb_open;
   uint32   flags;

   uint8    name[SERIAL_NAME_SIZE];
   uint16   irq;  /* 8bits should be enought on intel ?? */
   uint32   base_addr;
   int32    sense;   /* -1 = auto, 0 = active high, 1 = active low */

   /* dump from lirc */
   int      type;
   int      signal_pin;
   int      signal_pin_change;
   int      on;
   int      off;
   int      features;

   uint8    is_reading;
   sem_id   read_sem;
   sem_id   wait_data_sem;
   uint8    is_waiting_data;
} TS_SERIAL_device;

/*************************
 *       Variables       *
 *************************/
extern TS_SERIAL_device hardware[];
extern int type;

/******************
 *    Functions   *
 ******************/

/* file detect.c */
extern int detect(TS_SERIAL_device *pas_Devices);

/* file device.c */
extern status_t SERIAL_InitDeviceArray(TS_SERIAL_device *ps_DeviceArray, 
                                   TS_SERIAL_device *ps_DefaultValue);
extern status_t SERIAL_FiniDeviceArray(TS_SERIAL_device *ps_DeviceArray);
extern TS_SERIAL_device *SERIAL_NewDevice(
                                 TS_SERIAL_device *ps_DeviceArray);
extern void SERIAL_FreeDevice(TS_SERIAL_device *ps_Device);


#endif
